//
//  NSObject(MTLControlItemEnabler).h
//  TravelAdvisor
//
//  Created by narita on 07/10/15.
//  Copyright 2007 MindTools Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>

/*!
 コントロールの挙動を宣言的に記述する為のカテゴリ
 
NSWindowControllerのサブクラスやApplicationDelegateのようなレスポンダチェーン中で
UIのenable/disableを制御する手法を簡潔にするカテゴリ。
 
以下のように書いているif文の集合を、宣言的な表記で書けるようにする。
- (BOOL)validateUserInterfaceItem:(id<NSValidatedUserInterfaceItem>)anItem
{
    BOOL theResult = NO;

    SEL theSelector = [anItem action];

    if( theSelector == @selector(appDelegateAction1:) )
    {
         theResult = [self canAppDelegateAction1:anItem];
    }
    else if( theSelector == @selector(appDelegateAction2:) )
    {
         theResult = [self canAppDelegateAction2:anItem];
    }
    else if( theSelector == @selector(appDelegateAction3:) )
    {
         theResult = [self canAppDelegateAction3:anItem];
    }

	return theResult;
}

まず、validateUserInterfaceItemはユーティリティメソッドを使って以下のように書く
- (BOOL)validateUserInterfaceItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    return [self controlItemEnabler:anItem];
}

この表記で、自動的に"canXXXXX:"が呼ばれてif文の集合を書かなくてすむ。
後は、canAppDelegateAction1の中身を書くだけ。
 
- (BOOL) canAppDelegateAction1:(id)anItem
{
 return YES or NO;
}
 .
 .
 .
 .
 
*/

extern NSString* const MTLInterfaceItem_Can;

@interface NSObject (MTLControlItemEnabler)

/*!
 @method     updateEnableUserInterfaceFromItem
 @abstract   与えられたinItemのactionから"canAction:"を合成し呼び出し、その結果を返す
 @discussion 各種クラスのvalidateUserInterfaceItemの実装をサポートする
 
 */

- (BOOL) updateEnableUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)inItem;

/*!
 @method     controlItemEnabler:
 @abstract   レスポンダーチェーンで使うユーティリティーメソッド。
 @discussion 
 */
- (BOOL) controlItemEnabler:(NSObject<NSValidatedUserInterfaceItem>*)inItem;


/// State (NSMixedState, NSOffState, NSOnState)
extern NSString* const MTLInterfaceItem_StateOf;
- (void) updateStateOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// Image
- (void) updateImageOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// Title
- (void) updateTitleOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// Label
- (void) updateLabelOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;


/// attributedTitle
- (void) updateAttributedTitleOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// objectValue
- (void) updateObjectValueOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// stringValue
- (void) updateStringValueOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// attributedStringValue
- (void) updateAttributedStringValueOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;

/// toolTip
- (void) updateToolTipOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem;


// ToDo:ソースコードの整理
// ToDo:NSMatrixでのラジオボタンやsegmentedControlのどれが選ばれているかのEnabler作成
// PopupButton

// ToDo:以下のプロパティにも対応する事
/// IntValue
/// FloatValue
/// DoubleValue
/// IntegerValue

@end
