//
//  NSObject(MTLControlItemEnabler).m
//  TravelAdvisor
//
//  Created by narita on 07/10/15.
//  Copyright 2007 MindTools Inc. All rights reserved.
//

#import "NSObject(MTLControlItemEnabler).h"

// selectorにプレフィックスを追加して返す
SEL prefixedSelector(SEL inSelector, NSString* inPrefix)
{
    NSString* theSelectorString = NSStringFromSelector(inSelector);
    NSString* thePrefixedString = [inPrefix stringByAppendingString:
                                   [theSelectorString stringByReplacingCharactersInRange:NSMakeRange(0,1)
                                                                              withString:[[theSelectorString substringToIndex:1]
                                                                                          uppercaseString]]];
    return NSSelectorFromString(thePrefixedString);
}

SEL setterSelectorFromName(NSString* inName)
{    
    NSString* theResultString = [NSString stringWithFormat:@"set%@:",
                                 [inName stringByReplacingCharactersInRange:NSMakeRange(0,1)
                                                                 withString:[[inName substringToIndex:1]
                                                                             uppercaseString]]];

    return NSSelectorFromString(theResultString);
}


// todo:命名規則を検討する事
@implementation NSObject (MTLRuntimeUtilitys)

typedef enum
{
    propertyAttributeIsReadable  = 0x1,
    propertyAttributeIsWriteable = 0x2
} EPropertyAttribute;

- (EPropertyAttribute) propertyExistsAtName:(NSString *)inName
{
    EPropertyAttribute theResult   = 0x0;
    
    if( [self respondsToSelector:NSSelectorFromString(inName)] )
    {
        theResult |= propertyAttributeIsReadable;
    }

    if( [self respondsToSelector:setterSelectorFromName(inName)] )
    {
        theResult |= propertyAttributeIsWriteable;
    }
    
    return theResult;
}

@end

NSString* const MTLInterfaceItem_Can     = @"can";

@implementation NSObject (MTLControlItemEnabler)

- (BOOL) updateEnableUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)inItem
{
    typedef BOOL  (*methodFuncPtr)(id, SEL, id);

    BOOL          theResult   = YES;
    methodFuncPtr theFunction = nil;

    // canメソッドの存在確認
    SEL		theSelector = prefixedSelector([inItem action], MTLInterfaceItem_Can);

    if( [ self respondsToSelector:theSelector ] )
    {
        // canメソッドの実行
        theFunction = (methodFuncPtr)[ self methodForSelector:theSelector];
        theResult   = (*theFunction)(self, theSelector, inItem);
    }
    else if( [[self superclass] conformsToProtocol:@protocol(NSUserInterfaceValidations)] )
    {
        // 親クラスのメソッドを実行
        SEL  theValidateSelector = @selector(validateUserInterfaceItem:);
        
        theFunction = (methodFuncPtr)[ self methodForSelector:theValidateSelector];
        theResult   = (*theFunction)(self, theValidateSelector, inItem);
    }
    
	return theResult;
}

- (BOOL) controlItemEnabler:(NSObject<NSValidatedUserInterfaceItem>*)inItem
{
    BOOL theResult = YES;
    
    [self updateStateOfUserInterfaceFromItem:inItem];
    [self updateImageOfUserInterfaceFromItem:inItem];
    [self updateTitleOfUserInterfaceFromItem:inItem];
    [self updateLabelOfUserInterfaceFromItem:inItem];

    [self updateAttributedTitleOfUserInterfaceFromItem:inItem];
    [self updateObjectValueOfUserInterfaceFromItem:inItem];
    [self updateStringValueOfUserInterfaceFromItem:inItem];
    [self updateAttributedStringValueOfUserInterfaceFromItem:inItem];
    [self updateToolTipOfUserInterfaceFromItem:inItem];

    theResult = [self updateEnableUserInterfaceFromItem:inItem];
    
	return theResult;
}


/// State (NSMixedState, NSOffState, NSOnState)
NSString* const MTLInterfaceItem_StateOf = @"stateOf";
- (void) updateStateOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    SEL theSelector;
    
    // stateメソッドの存在確認
    theSelector = prefixedSelector([anItem action], MTLInterfaceItem_StateOf);
    
    if(
       [ self respondsToSelector: theSelector ]
       &&
       [anItem propertyExistsAtName:@"state"] & propertyAttributeIsWriteable
       )
    {
        typedef NSInteger (*methodFuncPtr)(id, SEL, id);

        // stateメソッドの実行
        methodFuncPtr theFunction;
        theFunction = (methodFuncPtr)[ self methodForSelector:theSelector];
        
        NSInteger theResult = (*theFunction)(self, theSelector, anItem);
                
        [anItem setValue:[NSNumber numberWithInteger:theResult]
                  forKey:@"state"];
    }
}

// 値がObject型の物
- (void) updatePropertyName:(NSString*) inName
      userInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*) inItem
{
    SEL theSelector;
    
    theSelector = prefixedSelector([inItem action],
                                   [inName stringByAppendingString:@"Of"]);
    
    if(
       [ self respondsToSelector:theSelector ]
       &&
       [inItem propertyExistsAtName:inName] & propertyAttributeIsWriteable
       )
    {
        [inItem setValue:[self performSelector:theSelector
                                    withObject:inItem]
                  forKey:inName];
    }
}


/// Image
- (void) updateImageOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"image"
       userInterfaceFromItem:anItem];
        
//    SEL theSelector;
//    
//    // imageメソッドの存在確認
//    theSelector = prefixedSelector([anItem action], @"image" @"Of");
//    
//    if(
//       [ self respondsToSelector:theSelector ]
//       &&
//       [anItem propertyExistsAtName:@"image"] & propertyAttributeIsWriteable
//       )
//    {        
//        // imageOfメソッドの実行結果をimageへ入力
//        [anItem setValue:[self performSelector:theSelector
//                                    withObject:anItem]
//                  forKey:@"image"];
//    }
}

/// Title
- (void) updateTitleOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"title"
       userInterfaceFromItem:anItem];
}

/// Label
- (void) updateLabelOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"label"
       userInterfaceFromItem:anItem];
}

/// attributedTitle
- (void) updateAttributedTitleOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"attributedTitle"
       userInterfaceFromItem:anItem];
}

/// objectValue
- (void) updateObjectValueOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"objectValue"
       userInterfaceFromItem:anItem];
}


/// stringValue
- (void) updateStringValueOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"stringValue"
       userInterfaceFromItem:anItem];
}


/// attributedStringValue
- (void) updateAttributedStringValueOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"attributedStringValue"
       userInterfaceFromItem:anItem];
}


/// toolTip
- (void) updateToolTipOfUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)anItem
{
    [self updatePropertyName:@"toolTip"
       userInterfaceFromItem:anItem];
}

@end

